-----------------------------------------------------------------------------------------------
-- Client Lua Script for CircleMeMaster
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "GuildLib"
require "GameLib"
require "ICCommLib"
require "ICComm"
require "ChatSystemLib"
 
-----------------------------------------------------------------------------------------------
-- CircleMeMaster Module Definition
-----------------------------------------------------------------------------------------------
local CircleMeMaster = {} 
local JSON = Apollo.GetPackage("Lib:dkJSON-2.5").tPackage
	

-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function CircleMeMaster:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- Set Category Default
	o.CatSelected = guilds
	-- Set Filter Defaults (All On to show everything)
	o.pvpFilter = true
	o.pveFilter = true
	o.rpFilter = true 
	o.otherFilter = true 
	o.twosFilter = true 
	o.threesFilter = true 
	o.fivesFilter = true
	o.dbPostList = {}
	o.dbMasterStorage = {}
	o.dbPersonalStorage = {}
	--o.charName = ""
	o.dbGroupTemplate = {
	Name = "",
	CatagoryType = nil,
	Announce = false,
	MemberInfo = {
		GuildLeader = "",
		MemberCount = 0,
		},
	Roles = { 
		pvp   = true,
		pve   = true,
		rp    = true,
		other = true,
	},
	ApplicationManager = "",
	Description = "",
	Stats = {
		Rating = 0,
		Wins = 0,
		Losses = 0,
		}
	}
	o.isBusy = false
	o.entryCache = {}
    return o
end

function CircleMeMaster:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
-----------------------------------------------------------------------------------------------
-- CircleMeMaster OnLoad
-----------------------------------------------------------------------------------------------
function CircleMeMaster:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("CircleMeMaster.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end
local function deepCopy(t)
    if type(t) == "table" then
        local copy = {}
        for k, v in next, t do
            copy[deepCopy(k)] = deepCopy(v)
        end
        return copy
    else
        return t
    end
end
function CircleMeMaster:OnRestore( eType, tSavedData )
    if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then return end
	
    -- Validate Save Version
    --if tSavedData.VersionNumber < VersionNumber then
        -- Rebuild Database based on version
   -- else 
        -- We will save all of the data for the account locally as to allow for importing of other characters data
        self.dbMasterStorage = deepCopy(tSavedData) or {}
		
   -- end
end
function CircleMeMaster:OnSave( eType )
    if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then return end
        tData = deepCopy(self.dbMasterStorage) -- Version number is on a per character basis, for abandoned characters 
    return tData
end 
-- Filters Functions
function CircleMeMaster:OnFilterCheck( wndHandler, wndControl, eMouseButton )
	local refName = wndControl:GetName() .. "Filter"
	self[refName] = wndControl:IsChecked()
	self:GenerateListByCat()
end

-----------------------------------------------------------------------------------------------
-- CircleMeMaster OnDocLoaded
-----------------------------------------------------------------------------------------------
function CircleMeMaster:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "CircleMeMasterForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
	    self.wndMain:Show(false, true)
		self.wndCfg = Apollo.LoadForm(self.xmlDoc, "ConfigWindow",nil,self)
		self.wndCfg:Show(false)
		Apollo.RegisterEventHandler("ReceivedMessageEvent","OnGlobalMessageReceived",self)
		Apollo.RegisterEventHandler("GuildRoster", "OnGuildRoster", self)
		Apollo.RegisterEventHandler("GuildMemberChange", "OnGuildRoster", self)
		self.myTimer = ApolloTimer.Create(1, false, "Connect_Global", self)
		self.SpamTimer = ApolloTimer.Create(600, true, "SpamTimerAnnounce", self)
		self.myTimer:Start()
		self.SpamTimer:Stop()
		Apollo.RegisterSlashCommand("cmm", "OnCircleMeMasterOn", self)
		
		self:toggleFilter()
		-- Do additional Addon initialization here
		self.CfgPos = "down"
		for _,v in pairs(GuildLib.GetGuilds()) do 
			v:RequestMembers()
		end
		--self.charName = GameLib.GetPlayerUnit():GetName()
		self:CleanUpGroupList() -- Cleanup List to remove my groups that I don't deal with
		-- Create The List
		
	end
	
end
 -- Cleanup Function 
function CircleMeMaster:CleanUpGroupList()
	
	local tCounterTable = {
		}
	
	-- Get Names of all of "My" groups 
	for idx, entry in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		local strName = entry.Name 
		local bPassedTest = false 
		for _,currGuild in pairs( GuildLib.GetGuilds() ) do 
			--Print("dbName: "..strName.." vs Guild Name: " .. currGuild:GetName())
			if currGuild:GetName() == strName then 
				--Print("Pass")
				if tCounterTable[""..strName..""] == nil then -- We only count the first one, remove the rest in-case of duplicates
					tCounterTable[""..strName..""] = 1 
					bPassedTest = true 
				elseif tCounterTable[""..strName..""] == 1 then 
					bPassedTest = false 
				end 
			end 
		end 
		if not bPassedTest then 
			--Print("Fail")
			self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""][idx] = nil 
		end 
	end
	tCounterTable = nil 
	self:GenerateGroupList()
	self:SetupConfigScreen()
end
-----------------------------------------------------------------------------------------------
-- CircleMeMaster Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/cmm"
function CircleMeMaster:OnCircleMeMasterOn(sCmd,sArg)
	if sArg == "" then 
		self.wndMain:Show(not self.wndMain:IsVisible() ) -- show the window
	else 
		if sArg == "update" then
			--Print("Sending Update message")
			self:Send( { action = "update", payload = {} } )
		else 
			Print("Invalid Command")
		end 
	end 
end 

function CircleMeMaster:OnCancel()
	self.wndMain:Show(false)
end 

function CircleMeMaster:OnGuildRoster(guildCurr, tRoster)
	-- Create The List
	self:GenerateGroupList()
	-- Get Guild Leader and update
	if tRoster == nil then return end
	if #self.dbMasterStorage == 0 then return end 
	for _,sName in pairs( tRoster ) do 
		if sName.nRank == 1 then 
			for idx, GroupList in pairs( self.dbMasterStorage ) do
				GroupList.MemberInfo.GuildLeader = sName.strName
			end 
		end 
	end 
end

function CircleMeMaster:GroupExists( strGroupName, dbSelect ) -- 1 = dbMasterStorage 2 = dbGroupList 
	if dbSelect == 1 then 
		for _,v in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
			--Print("Looking For"..v.Name)
			if v.Name == strGroupName then return true end 
		end
		return false 
	elseif dbSelect == 2 then 
		for _,v in pairs( self.dbGroupList[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
			if v.Name == strGroupName then return true end
		end 
		return false 
	end 
end

function CircleMeMaster:UpdateGroup( strGroupName, tData, dbSelect )
	if dbSelect == 1 then 
		for _,v in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
			if v.Name == strGroupname then 
				v.CatagoryType = tData.CatagoryType
				v.MemberInfo = {
					GuildLeader = tData.MemberInfo.GuildLeader,
					MemberCount = tData.MemberInfo.MemberCount,
				}
				Roles = {
					pvp = tData.Roles.pvp,
					pve = tData.Roles.pve,
					rp = tData.Roles.rp,
					other = tData.Roles.other,
					}
				v.ApplicationManager = tData.ApplicationManager
				if v.CatagoryType >= 3 then 
					v.Stats = {
						Rating = tData.Stats.Rating,
						Wins = tData.Stats.Wins,
						Losses = tData.Stats.Losses,
					}
				end 
			end 
		end 
	elseif dbSelect == 2 then 
		for _,v in pairs( self.dbGroupList ) do 
			if v.Name == strGroupname then 
				v.CatagoryType = tData.CatagoryType
				v.MemberInfo = {
					GuildLeader = tData.MemberInfo.GuildLeader,
					MemberCount = tData.MemberInfo.MemberCount,
				}
				Roles = {
					pvp = tData.Roles.pvp,
					pve = tData.Roles.pve,
					rp = tData.Roles.rp,
					other = tData.Roles.other,
					}
				v.ApplicationManager = tData.ApplicationManager
				if v.CatagoryType >= 3 then 
					v.Stats = {
						Rating = tData.Stats.Rating,
						Wins = tData.Stats.Wins,
						Losses = tData.Stats.Losses,
					}
				end 
			end 
		end 
	end 
end
function CircleMeMaster:RemoveGroup( strGroupName )
	for idx, data in ipairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		if data[idx].Name == strGroupName then 
			self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""][idx] = nil
			return 
		end 
	end 
end 
function CircleMeMaster:GenerateGroupList()
	if self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] == nil then self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] = {} end 

	-- deal with Cache Items First 
	if self.entryCache ~= nil and #self.entryCache > 0 then 
		for _,v in pairs( self.entryCache ) do 
			if self:GroupExists( v.Name, 1 ) == false then 
				table.insert( self.dbMasterStorage, v )
				
			end 
		end 
	end 
	self.isBusy = false 
	
	-- GuildTypes: Guild = 1, Circle = 2, Warplot = 3, 2v2 Arena Team = 4, 3v3 Arena Team = 5, 5v5 Arena Team = 6
	local tTempEntry = {}
	for _, tCurrGuild in pairs( GuildLib:GetGuilds() ) do 
		if tCurrGuild:GetType() == 3 then
		
		else 
			tTempEntry = deepCopy(self.dbGroupTemplate)
			if tCurrGuild:GetPvpRatings() == nil then
			tTempEntry.MemberInfo = {
				GuildLeader = self:FindGuildLeader() or "Unknown",
				MemberCount = tCurrGuild:GetMemberCount() or 0,
			}
			else 
			tTempEntry.Stats = {
				Rating = tCurrGuild:GetPvpRatings().Rating or 0,
				Wins = tCurrGuild:GetPvpRatings().Wins or 0,
				Losses = tCurrGuild:GetPvpRatings().Losses or 0,
			}			
			end 

			-- All Groups Get The Following
			tTempEntry.Name = tCurrGuild:GetName()
			tTempEntry.CatagoryType = tCurrGuild:GetType()
			tTempEntry.ApplicationManager = GameLib.GetPlayerUnit():GetName()
			local match = false 
			for _,val in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
				if val.Name == tTempEntry.Name then
					match = true
					-- we found it, so lets update it instead
					val.Roles = tTempEntry.Roles
					val.MemberInfo = tTempEntry.MemberInfo
					val.ApplicationManager = tTempEntry.ApplicationManager
				end 
			end 
			if match == false then 
				table.insert(self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""], tTempEntry)
			end 
		end 
	end 
	self:RefreshCounts()
end 

function CircleMeMaster:FindGuildLeader()
	local sLeaderName = ""
	return sLeaderName
end
-- Chat Management 
function CircleMeMaster:SpamTimerAnnounce()
	--if self.dbPostList == nil then self:UpdateMyGuilds() end
	if #self.dbPostList == 0 then return end -- No Entries At All
	
	-- Get All Possible Entries
	local tempList = {}
	for _, entry in pairs(self.dbPostList) do
		if entry.announce == true then 
			table.insert(tempList,entry)
		end 
	end 
	
	local maxNum = #tempList
	if maxNum == 0 then return end -- Nothing Listed
	local randEntry = math.random(maxNum)
	local a = 1
	for _,group in pairs( tempList ) do
		if a == randEntry then 
			for _,channel in pairs(ChatSystemLib.GetChannels()) do
				if channel:GetType() == ChatSystemLib.ChatChannel_Zone then
					channel:Send("<"..group.name.."> "..group.desc.." - Dislike the message? Get CircleMeMaster Addon!")
				end
			end
		--Print("<"..group.name.."> "..group.desc.." - Dislike the message? Get CircleMeMaster Addon!")
		end 
		a = a + 1
	end
	
	a = nil
	tempList = nil
end 
function CircleMeMaster:ToggleSpamAnnounce( wndHandler, wndControl, eMouseButton )
	if SpamStarted == false then 
		if #self.dbPostList == 0 then Print("Nothing To Post... Halting") self.SpamTimer:Stop() return end
		SpamStarted = true 
		wndControl:SetText("On")
		self.SpamTimer:Start()
	else 
		SpamStarged = false 
		wndControl:SetText("Off")
		self.SpamTimer:Stop()
	end 
end	
function CircleMeMaster:AnnounceArrival()
	self:Send( { action = "update" } )
end
-----------------------------------------------------------------------------------------------
-- Channel (ICCommLib) Functions
-----------------------------------------------------------------------------------------------
function CircleMeMaster:Connect_Global()
	local joined = false
	
	if self.CMMGlobal then
		joined = true 
	end

	if joined then 
		self.myTimer:Stop()
		return
	else
		self.CMMGlobal = ICCommLib.JoinChannel("CircleMeMaster")
		if self.CMMGlobal then
			self.CMMGlobal:SetJoinResultFunction("ChannelJoin", self)
			self.CMMGlobal:SetReceivedMessageFunction("OnGlobalMessageReceived", self)
			self:AnnounceArrival()
			self:SendListToChannel()
		end
	end 
end
function CircleMeMaster:Send(tMessage)
	local strMsg = JSON.encode(tMessage)
	self.CMMGlobal:SendMessage(tostring(strMsg))
end
function CircleMeMaster:RefreshCounts()
	local nGuilds = 0
	local nCircles = 0 
	local nArenaTeams = 0 
	if #self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] > 0 then 
		for _,v in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
			if v.CatagoryType == 1 then 
				nGuilds = nGuilds + 1 
			elseif v.CatagoryType == 2 then 
				nCircles = nCircles + 1 
			elseif v.CatagoryType >= 4 then 
				nArenaTeams = nArenaTeams + 1 
			end 
		end 
	end 
	self.wndMain:FindChild("UpperMenu"):FindChild("GuildsBtn"):SetText("Guilds ("..nGuilds..")")
	self.wndMain:FindChild("UpperMenu"):FindChild("CirclesBtn"):SetText("Circles ("..nCircles..")")
	self.wndMain:FindChild("UpperMenu"):FindChild("ArenaTeamsBtn"):SetText("Arena Teams ("..nArenaTeams..")")
end 
function CircleMeMaster:OnGlobalMessageReceived(channel, strMessage, strSender)
	local strMessage = JSON.decode(strMessage)
	--Print("I got a channel message: ")
	if strMessage.action == "insert" then
		--Print("Received")
		if self:GroupExists( strMessage.payload.Name, 1 ) then return end 
		
		if strMessage.payload.entryType ~= nil then 
			Print("Entry From old System") -- IGNORE OLD MESSAGES PLZKTHXBAI
		else
			--Print("insert From "..strSender)
			table.insert( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""], strMessage.payload )
			self:RefreshCounts()
		end 
		
			-- Cache System
		if self.isBusy == true then
			table.insert( self.entryCache, strMessage.payload )
			return
		end
		self.isBusy = true 
		
		elseif strMessage.action == "update" then 
		--Print("I got an update Message")
		self:SendListToChannel() -- Uncommenting This Will make People get the new version hopefully ;s
	elseif strMessage.action == "apply" then 
		if strMessage.ApplicationManager == GameLib.GetPlayerUnit():GetName() then 
			Print("[CMM] You have received a message from [ "..strSender.." ] Regarding an Application to [ ".. strMessage.Name .." ]")
		end
	end 
	
end

function CircleMeMaster:SendListToChannel()
	-- My lists bring all Exiles to the Yard. Damn Right. It's Bigger Than Yours!
	self.dbPostList = {}
	if self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] == nil then self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] = {} end 
	self:GenerateGroupList()
	if #self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] == 0 then return end
	self:RefreshCounts()
	for _,entry in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		if entry.Announce == true then -- It Wants To Be Known?
			--Print("Announce True For: "..entry.Name)
			if entry.ApplicationManager == nil then -- Old Format Need Not Apply
				Print("[CMM] - Please Update Your Version Of CircleMeMaster")
			elseif entry.ApplicationManager == GameLib.GetPlayerUnit():GetName() then -- Are We The Admin For It?
					table.insert( self.dbPostList, entry )
			end
		else 
			--Print("Announce False for: "..entry.Name)
		end 
	end 
	--Print("List Size: "..#self.dbPostList)
	-- Don't Let Your Dreams Be Dreams! DO IT!
	for _,val in pairs( self.dbPostList ) do 
		self:Send( { action = "insert", payload = val } )
	end 
end
function CircleMeMaster:ThrottledEvent(iccomm, strSender, idMessage)
	if iccomm:GetId() == self.CMMGlobal:Getid() then
		Print("[CMM] - Message has been throttled...")
		return
	end
end
-- List Management
function CircleMeMaster:AddItemToList( grdList, tData )
	--local tTime = os.date("*t", os.time() )
	--local sDate = tTime.day .. "/" .. tTime.month .. "/" .. tTime.year .. " - " .. tTime.hour .. ":" .. tTime.min
	local iCurrRow = grdList:AddRow("")
	local sName = ""
	if tData.Name == nil then 
		sName = tData.name
	else
		sName = tData.Name 
	end 	
	grdList:SetCellText( iCurrRow, 1, string.format("%s",sName) )
end
function CircleMeMaster:OnListItemSelected(wndControl, wndHandler, iRow, iCol,iCurrRow, iCurrCol)
	local strName = self.wndMain:FindChild("grdGroupList"):GetCellText(iRow,iCol)
	local tEntry = {}
	for _,k  in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		if k.Name == strName then 
			tEntry = k
		end 
	end 
	local cfgWnd = self.wndMain:FindChild("ContentWindow")
	local txtDesc = cfgWnd:FindChild("txtDescription")
	local cName = cfgWnd:FindChild("txtName")
	if tEntry.CatagoryType == nil then return end 
	if tEntry.CatagoryType == 1 or tEntry.CatagoryType == 2 then 
		local btnPvP = cfgWnd:FindChild("pvp")
		local btnPvE = cfgWnd:FindChild("pve")
		local btnRP  = cfgWnd:FindChild("rp")
		local btnOther = cfgWnd:FindChild("other")
		local btnTwos = cfgWnd:FindChild("twos")
		local btnThrees = cfgWnd:FindChild("threes")
		local btnFives = cfgWnd:FindChild("fives")
		if tEntry.Roles.pvp then btnPvP:SetTextColor("Green") else btnPvP:SetTextColor("Red") end
		if tEntry.Roles.pve then btnPvE:SetTextColor("Green") else btnPvE:SetTextColor("Red") end
		if tEntry.Roles.rp then btnRP:SetTextColor("Green") else btnRP:SetTextColor("Red") end
		if tEntry.Roles.other then btnOther:SetTextColor("Green") else btnOther:SetTextColor("Red") end
		btnTwos:SetTextColor("Grey")
		btnThrees:SetTextColor("Grey") 
		btnFives:SetTextColor("Grey")
	elseif tEntry.CatagoryType >= 4 then 
		local btnTwos = cfgWnd:FindChild("twos")
		local btnThrees = cfgWnd:FindChild("threes")
		local btnFives = cfgWnd:FindChild("fives")
		local btnPvP = cfgWnd:FindChild("pvp")
		local btnPvE = cfgWnd:FindChild("pve")
		local btnRP  = cfgWnd:FindChild("rp")
		local btnOther = cfgWnd:FindChild("other")
		-- local adminName = cfgWnd:FindChild("txtAdmin")
		--adminName:SetText( "Contact: "..tEntry.admin )
		if tEntry.CatagoryType == 4 then btnTwos:SetTextColor("Green") else btnTwos:SetTextColor("Red") end
		if tEntry.CatagoryType == 5 then btnThrees:SetTextColor("Green") else btnThrees:SetTextColor("Red") end
		if tEntry.CatagoryType == 6 then btnFives:SetTextColor("Green") else btnFives:SetTextColor("Red") end
		btnPvP:SetTextColor("Grey")
		btnPvE:SetTextColor("Grey")
		btnRP:SetTextColor("Grey")
		btnOther:SetTextColor("Grey")
	end
	txtDesc:SetText(tEntry.Description)
	cName:SetText("Name: " .. tEntry.Name)
end
--Cat Selection
function CircleMeMaster:OnCatSelected( wndHandler, wndControl, eMouseButton )
	if wndControl:GetName() == "GuildsBtn" then self.CatSelected = 1
	elseif wndControl:GetName() == "CirclesBtn" then self.CatSelected = 2 
	elseif wndControl:GetName() == "ArenaTeamsBtn" then self.CatSelected = 3 
	end 
	self:GenerateGroupList()
	self:GenerateListByCat()
end
function CircleMeMaster:toggleFilter()
	-- Turn on all filter checkboxes: Wish we had a GetClass() option :s
	for _,v in pairs(self.wndMain:FindChild("wndGroupFilters"):FindChild("arena"):GetChildren()) do 
		if v:GetName() ~= "txtRating" and v:GetName() ~= "rating" then 
			if not v:IsChecked() then
				v:SetCheck(true)
			end 
		end
	end 
	for _,v in pairs(self.wndMain:FindChild("wndGroupFilters"):FindChild("circles"):GetChildren()) do 
		if not v:IsChecked() then
			v:SetCheck(true)
		end 
	end 
	for _,v in pairs(self.wndMain:FindChild("wndGroupFilters"):FindChild("guilds"):GetChildren()) do 
		if not v:IsChecked() then
			v:SetCheck(true)
		end 
	end 
end 

function CircleMeMaster:GenerateListByCat()
	self.tWndRefs = {}
	self.tWndRefs.List = self.wndMain:FindChild("grdGroupList")
	-- Clear Grid
	self.tWndRefs.List:DeleteAll()
	self.tWndRefs.Filters = self.wndMain:FindChild("wndGroupFilters")
	self.tWndRefs.ContentWindow = self.wndMain:FindChild("ContentWindow")
	self:toggleFilter(self.CatSelected)
	-- filters
	-- Toggle only selected catagory filters
	-- for _, obj in pairs( self.wndMain:FindChild("wndGroupFilters"):GetChildren() ) do 
		-- local wndName = obj:GetName()
		-- if self.CatSelected == 1 then
			-- if wndName == "guilds" then
				-- obj:Show(true)
			-- else 
				-- obj:Show(false)
			-- end 
		-- elseif self.CatSelected == 2 then 
			-- if wndName == "circles" then 
				-- obj:Show(true)
			-- else 
				-- obj:Show(false)
			-- end
		-- elseif self.CatSelected == 3 then 
			-- if wndName == "arena" then 
				-- obj:Show(true)
			-- else
				-- obj:Show(false)
			-- end 
		-- end 
	-- end 
	
	for idx, data in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		-- Compare Filters 
		local gtg = true
		-- Validate that we are the admin of the one being listed.
		if data.ApplicationManager ~= GameLib.GetPlayerUnit():GetName() then gtg = false end 
		if data.CatagoryType == self.CatSelected then 
			--if self:FilterCheck( data.Roles ) then 
				self:AddItemToList( self.tWndRefs.List, data )
			--end 
		elseif self.CatSelected == 3 then -- PvP List -> CatalogType 4,5,6 
			if data.CatagoryType >= 4 then 
				--if self:FilterCheck( { CatagoryType = data.CatagoryType, Stats = data.Stats } ) then 
					self:AddItemToList( self.tWndRefs.List, data )
				--end 
			end 
		end 
	end 
end 
-- Filter Functions
function CircleMeMaster:FilterCheck( tFilters, rating ) -- Rating added as the only entry checkbox. At end so it doesn't matter if it is passed or not.
	if (self.CatSelected == 1) or (self.CatSelected == 2) then 
		-- guilds and circles
		if (tFilters.pvp == self.pvpFilter) and (tFilters.pve == self.pveFilter) and (tFilters.rp == self.rpFilter) and (tFilters.other == self.otherFilter) then
			return true
		else
			return false 
		end 
	elseif type(rating) == "table" then -- Only the PvP Gets a table sent to this variable
		local ratingsPass = false 
		if self.wndMain:FindChild("wndGroupFilters"):FindChild("arena"):FindChild("rating"):IsChecked() then 
			if ( tonumber(self.wndMain:FindChild("wndGroupFilters"):FindChild("arena"):FindChild("txtRating"):GetText()) <= rating.Stats.Rating) then 
				ratingsPass = true
			end 
		else
			ratingsPass = true 
		end
		if ratingsPass == true then 
			if tFilters.CatagoryType == 4 then
				if self.twosFilter == true then 
					return true 
				end
			end 
			if tFilters.CatagoryType == 5 then 
				if self.threesFilter == true then
					return true 
				end 
			end 
			if tFilters.CataGoryType == 6 then 
				if self.fivesFilter == true then 
					return true 
				end 
			end 
		end 
		return false 
	end 
end

-- Configuration Functions 

-- SlideConfigToggle
function CircleMeMaster:ToggleSlideConfigWindow(wndHandler, wndControl, eMouseButton )
	if not self.CfgWndTimer then self.CfgWndTimer = ApolloTimer.Create(0.01, true, "CfgWindowSliderTimer", self) self.CfgWndTimer:Stop() end 
	local a,b,c,d = self.wndMain:GetAnchorOffsets()
	a = (a+16)
	b = (b+62)
	c = (c-16)
	d = (d-45)
	self.PositionUpOffsets = { a,b,c,d } -- up
	b = b+530
	d = d+530
	self.PositionDownOffsets = { a,b,c,d } -- down
	if self.CfgPos == "down" then 
		self.wndCfg:SetAnchorOffsets( self.PositionDownOffsets[1],self.PositionDownOffsets[2],self.PositionDownOffsets[3],self.PositionDownOffsets[4] )
		self.PositionCurrentOffsets = { self.PositionDownOffsets[1],self.PositionDownOffsets[2],self.PositionDownOffsets[3],self.PositionDownOffsets[4] }
	elseif self.CfgPos == "up" then 
		self.wndCfg:SetAnchorOffsets(self.PositionUpOffsets[1],self.PositionUpOffsets[2],self.PositionUpOffsets[3],self.PositionUpOffsets[4])
		self.PositionCurrentOffsets = {self.PositionUpOffsets[1],self.PositionUpOffsets[2],self.PositionUpOffsets[3],self.PositionUpOffsets[4]}
	end 
	
	self.wndCfg:Show(true)
	self.wndCfg:ToFront()
	self.CfgWndTimer:Start()
end 

function CircleMeMaster:CfgWindowSliderTimer()
	if self.CfgPos == nil then self.CfgPos = "down" end 
	local mod = math.floor( (self.PositionUpOffsets[2]-self.PositionDownOffsets[2]) / 8 ) -- 8 is the number of 'slides' we want to transition with
	if self.CfgPos == "down" then
		self.wndMain:FindChild("btnCfgToggle"):Show(false)
		self.PositionCurrentOffsets[2] = self.PositionCurrentOffsets[2] + mod
		self.PositionCurrentOffsets[4] = self.PositionCurrentOffsets[4] + mod
		if (self.PositionCurrentOffsets[2] < self.PositionUpOffsets[2]) then 
			self.wndCfg:SetAnchorOffsets( self.PositionUpOffsets[1], self.PositionUpOffsets[2], self.PositionUpOffsets[3], self.PositionUpOffsets[4])
			self.CfgWndTimer:Stop()
			self.CfgPos = "up"
			self.wndCfg:Show(true)
			self.wndCfg:FindChild("btnCfgToggle1"):Show(true)
			self.wndCfg:FindChild("btnCfgToggle1"):ToFront()
			self.wndMain:Show(false)
			return
		end 
		self.wndCfg:SetAnchorOffsets(self.PositionCurrentOffsets[1],self.PositionCurrentOffsets[2],self.PositionCurrentOffsets[3],self.PositionCurrentOffsets[4])
	elseif self.CfgPos == "up" then 
		self.wndCfg:FindChild("btnCfgToggle1"):Show(false)
		self.PositionCurrentOffsets[2] = self.PositionCurrentOffsets[2] - mod 
		self.PositionCurrentOffsets[4] = self.PositionCurrentOffsets[4] - mod 
		if ( self.PositionCurrentOffsets[2] > self.PositionDownOffsets[2] ) then 
			self.wndCfg:SetAnchorOffsets(self.PositionDownOffsets[1], self.PositionDownOffsets[2], self.PositionDownOffsets[3], self.PositionDownOffsets[4])
			self.CfgWndTimer:Stop()
			self.CfgPos = "down"
			self.wndCfg:Show(false)
			self.wndMain:FindChild("btnCfgToggle"):Show(true)
			self.wndMain:FindChild("btnCfgToggle"):ToFront()
			self.wndMain:Show(true)
			return
		end 
		self.wndCfg:SetAnchorOffsets(self.PositionCurrentOffsets[1],self.PositionCurrentOffsets[2],self.PositionCurrentOffsets[3],self.PositionCurrentOffsets[4])
	end 
end 
-- Cfg Screen
function CircleMeMaster:SetupConfigScreen()
	self.wndCfg:FindChild("cfgGroupContainer")
	local nCircles = 2 
	local nArenas = 7
	for _,v in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		if v.CatagoryType == 1 then 
			self.wndCfg:FindChild("cfgGroupContainer"):FindChild("Button1"):SetText(v.Name)
			self.wndCfg:FindChild("cfgGroupContainer"):FindChild("Button1"):SetData( v )
		end 
		if v.CatagoryType == 2 then 
			self.wndCfg:FindChild("cfgGroupContainer"):FindChild("Button"..nCircles):SetText(v.Name)
			self.wndCfg:FindChild("cfgGroupContainer"):FindChild("Button"..nCircles):SetData( v )
			nCircles = nCircles + 1
		end 
		if ( (v.CatagoryType == 4) or (v.CatagoryType == 5) or (v.CatagoryType == 6) ) then 
			local aName = ""
			if v.CatagoryType == 4 then aName = "2v2"
			elseif v.CatagoryType == 5 then aName = "3v3"
			elseif v.CatagoryType == 6 then aName = "5v5"
			end 
			self.wndCfg:FindChild("cfgGroupContainer"):FindChild("Button"..nArenas):SetText("["..aName.."] " ..v.Name)
			self.wndCfg:FindChild("cfgGroupContainer"):FindChild("Button"..nArenas):SetData( v )
			nArenas = nArenas + 1
		end 
	end
	-- Reset Display Entries
	self.wndCfg:FindChild("btnAnnounce"):SetCheck(false)
	self.wndCfg:FindChild("txtName"):SetText("")
	self.wndCfg:FindChild("txtDescription"):SetText("")
	
end
-- Cfg Buttons
function CircleMeMaster:OnCfgButtonClick( wndHandler, wndControl, eMouseButton ) -- Main window bottom button (cogwheel)
	-- every button calls the same function, get data to determine what to do
	local btnData = wndControl:GetData()
	if btnData == nil then return end -- nothing selected
	local groupName
	if btnData.Name == nil then 
		groupName = wndControl:GetText()
	else
		groupName = btnData.Name
	end 
	if btnData.CatagoryType == 1 or btnData.CatagoryType == 2 then 
		-- CheckBoxes for Roles
		self.wndCfg:FindChild("btnPvp"):Show(true)
		self.wndCfg:FindChild("btnPve"):Show(true)
		self.wndCfg:FindChild("btnRp"):Show(true)
		self.wndCfg:FindChild("btnOther"):Show(true)
		self.wndCfg:FindChild("btnPvp"):SetCheck(btnData.Roles.pvp)
		self.wndCfg:FindChild("btnPve"):SetCheck(btnData.Roles.pve)
		self.wndCfg:FindChild("btnRp"):SetCheck(btnData.Roles.rp)
		self.wndCfg:FindChild("btnOther"):SetCheck(btnData.Roles.other)
	else -- set to off and hide
		self.wndCfg:FindChild("btnPvp"):Show(false)
		self.wndCfg:FindChild("btnPve"):Show(false)
		self.wndCfg:FindChild("btnRp"):Show(false)
		self.wndCfg:FindChild("btnOther"):Show(false)
		self.wndCfg:FindChild("btnPvp"):SetCheck(false)
		self.wndCfg:FindChild("btnPve"):SetCheck(false)
		self.wndCfg:FindChild("btnRp"):SetCheck(false)
		self.wndCfg:FindChild("btnOther"):SetCheck(false)
	end 
	self.wndCfg:FindChild("txtDescription"):SetText( btnData.Description )
	self.wndCfg:FindChild("txtName"):SetText(btnData.Name )
	self.wndCfg:FindChild("btnAnnounce"):SetCheck(btnData.Announce)
end
function CircleMeMaster:OnCfgSaveButtonClick( wndHandler, wndControl, eMouseButton ) -- CheckBoxes
	-- name of group
	local strName = self.wndCfg:FindChild("txtName"):GetText()
	for _,v in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		--Print("Comp1: "..strName.." vs. ".. v.Name)
		if v.Name == strName then 
			-- Get Variables and assign them to v 
			v.Description = self.wndCfg:FindChild("txtDescription"):GetText()
			v.Roles.pvp = self.wndCfg:FindChild("btnPvp"):IsChecked()
			v.Roles.pve = self.wndCfg:FindChild("btnPve"):IsChecked()
			v.Roles.rp = self.wndCfg:FindChild("btnRp"):IsChecked()
			v.Roles.other = self.wndCfg:FindChild("btnOther"):IsChecked()
			v.Announce = self.wndCfg:FindChild("btnAnnounce"):IsChecked()
			return 
		end 
	end 	
	self:GenerateGroupList()
	self:SetupConfigScreen()
end
function CircleMeMaster:OnApplyButtonClick( wndHandler, wndControl, eMouseButton ) 
	local strName = self.wndMain:FindChild("grdGroupList"):GetCellText(self.wndMain:FindChild("grdGroupList"):GetCurrentRow(),1)
	local tEntry = {}
	for _,k  in pairs( self.dbMasterStorage[""..GameLib.GetPlayerUnit():GetName()..""] ) do 
		if k.Name == strName then 
			tEntry = k
		end 
	end 
	self:Send( { action = "apply", Name = tEntry.Name, ApplicationManager = tEntry.ApplicationManager } )
end
-----------------------------------------------------------------------------------------------
-- CircleMeMaster Instance
-----------------------------------------------------------------------------------------------
local CircleMeMasterInst = CircleMeMaster:new()
CircleMeMasterInst:Init()
